@@ -1,11 +1,10 @@
+/// PTD, or [post type discovery](https://indieweb.org/ptd) is a means of resolving the
+/// semantically relevant post type of a provided [microformats::types::Item].
 use std::collections::HashMap;
 use std::iter::FromIterator;
 use std::str::FromStr;
 
-fn default_h_value() -> String {
-    "entry".to_string()
-}
-
+/// A canonical list of the recognized post types.
 #[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
 #[serde(rename_all = "kebab-case")]
 pub enum Type {
@@ -64,24 +63,37 @@ impl Default for Type {
 #[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
 #[serde(untagged, rename_all = "kebab-case")]
 pub enum PostType {
+    /// Represents a simpler (textual) form of a post type.
     Simple(Type),
+
+    /// Represents an expanded way to describe a post type.
     Expanded {
+        /// The presentational name of the post type.
         name: String,
 
+        /// The known post type being expanded.
         #[serde(alias = "type")]
         kind: Type,
 
-        #[serde(default = "default_h_value")]
-        h: String,
+        /// The container type represented as a [microformats::types::Class]
+        #[serde(default = "default_class")]
+        h: microformats::types::Class,
 
+        /// Recognized properties for this post type.
         #[serde(default)]
         properties: Vec<String>,
 
+        /// Properties for this post type that are required.
         #[serde(default)]
         required_properties: Vec<String>,
     },
 }
 
+fn default_class() -> microformats::types::Class {
+    use crate::mf2;
+    mf2::types::Class::Known(mf2::types::KnownClass::Entry)
+}
+
 impl From<PostType> for Type {
     fn from(post_type: PostType) -> Type {
         match post_type {
@@ -159,33 +171,46 @@ fn properties_from_type() -> HashMap<String, Type> {
     )
 }
 
-/// Attempts to determine the types discovered by this item.
+/// Aims to resolve the known post type of the provided [microformats::types::Item].
+///
+/// ```
+/// # use microformats::algorithms::ptd::*;
+/// # use serde_json::*;
+///
+/// resolve_from_object(json!({
+///     "properties": {
+///         "like-of": ["https://indieweb.org/like"]
+///     }
+/// }))
+/// ```
 pub fn resolve_from_object<V>(into_item_mf2: V) -> Option<Type>
 where
     V: TryInto<microformats::types::Item>,
 {
     if let Ok(item_mf2) = into_item_mf2.try_into() {
-    let props = item_mf2.properties.borrow();
-    let mut types: Vec<Type> = vec![];
-    let has_content = props.contains_key(&"content".to_owned());
-    let has_name = props.contains_key(&"name".to_owned());
+        let props = item_mf2.properties.borrow();
+        let mut types: Vec<Type> = vec![];
+        let has_content = props.contains_key(&"content".to_owned());
+        let has_name = props.contains_key(&"name".to_owned());
 
-    properties_from_type().iter().for_each(|(key, val)| {
-        if props.contains_key(&key.to_owned()) {
-            types.push(val.clone());
+        properties_from_type().iter().for_each(|(key, val)| {
+            if props.contains_key(&key.to_owned()) {
+                types.push(val.clone());
+            }
+        });
+
+        if has_name && has_content {
+            types.push(Type::Article)
+        } else if !has_name && has_content {
+            types.push(Type::Note)
         }
-    });
 
-    if has_name && has_content {
-        types.push(Type::Article)
-    } else if !has_name && has_content {
-        types.push(Type::Note)
+        combinatory_type(types.clone())
+            .or(types.first().cloned())
+            .or(Some(Type::Note))
+    } else {
+        None
     }
-
-    combinatory_type(types.clone())
-        .or(types.first().cloned())
-        .or(Some(Type::Note))
-    } else { None }
 }
 
 fn combinatory_type(types: Vec<Type>) -> Option<Type> {
@@ -230,7 +255,7 @@ mod tests {
             Some(PostType::Expanded {
                 name: "Note".to_string(),
                 kind: Type::Note,
-                h: "entry".to_string(),
+                h: default_class(),
                 properties: Vec::default(),
                 required_properties: Vec::default()
             })
@@ -250,7 +275,7 @@ mod tests {
                 PostType::Expanded {
                     name: "Note".to_string(),
                     kind: Type::Note,
-                    h: "entry".to_string(),
+                    h: default_class(),
                     properties: Vec::default(),
                     required_properties: Vec::default()
                 },
@@ -266,7 +291,7 @@ mod tests {
             PostType::Expanded {
                 kind: Type::Like,
                 name: "Like".to_string(),
-                h: "entry".to_string(),
+                h: default_class(),
                 properties: Vec::default(),
                 required_properties: Vec::default()
             }
