@@ -71,7 +71,7 @@ pub fn for_url(url: &str, rel: &str) -> Result<Vec<String>, crate::Error> {
         body: vec![],
     };
 
-    req.send("get").and_then(|resp| {
+    req.send("GET").and_then(|resp| {
         let rels_from_header = get_rels_from_header(&resp.headers, rel);
         let rels_from_body = get_rels_from_body(resp.body_as_string()?, rel);
 
