@@ -2,7 +2,11 @@ use crate::algorithms::ptd::{PostType, Type};
 use crate::algorithms::Properties;
 use crate::client::{self, ResponseValue};
 use crate::indieauth::AccessToken;
-use serde::{Deserialize, Serialize};
+use crate::mf2::types::Item;
+use serde::de::{self, Visitor};
+use serde::ser::SerializeSeq;
+use serde::{Deserialize, Deserializer, Serialize};
+use std::fmt;
 use std::str::FromStr;
 use std::string::ToString;
 
@@ -57,15 +61,13 @@ impl Default for Order {
     }
 }
 
-impl From<String> for Order {
-    fn from(v: String) -> Self {
-        let lv = v.to_lowercase();
-        if lv == "descending" || lv == "desc" {
-            Self::Descending
-        } else if lv == "ascending" || lv == "asc" {
-            Self::Ascending
-        } else {
-            Self::Descending
+impl FromStr for Order {
+    type Err = std::convert::Infallible;
+
+    fn from_str(v: &str) -> Result<Self, Self::Err> {
+        match v.to_lowercase().as_str() {
+            "ascending" | "asc" | "a" => Ok(Self::Ascending),
+            "descending" | "desc" | "d" | _ => Ok(Self::Descending),
         }
     }
 }
@@ -93,7 +95,8 @@ impl<'de> Deserialize<'de> for Order {
     where
         D: serde::Deserializer<'de>,
     {
-        String::deserialize(deserializer).map(|vs| vs.into())
+        <&str as serde::Deserialize>::deserialize(deserializer)
+            .and_then(|s| Self::from_str(s).map_err(de::Error::custom))
     }
 }
 
@@ -110,17 +113,14 @@ impl Default for Visibility {
     }
 }
 
-impl From<String> for Visibility {
-    fn from(v: String) -> Self {
-        let lv = v.to_lowercase();
-        if lv == "public" {
-            Visibility::Public
-        } else if lv == "private" {
-            Visibility::Private
-        } else if lv == "unlisted" {
-            Visibility::Unlisted
-        } else {
-            Visibility::Public
+impl FromStr for Visibility {
+    type Err = std::convert::Infallible;
+
+    fn from_str(v: &str) -> Result<Self, Self::Err> {
+        match v.to_lowercase().as_str() {
+            "private" => Ok(Visibility::Private),
+            "unlisted" => Ok(Visibility::Unlisted),
+            "public" | _ => Ok(Visibility::Public),
         }
     }
 }
@@ -149,102 +149,14 @@ impl<'de> Deserialize<'de> for Visibility {
     where
         D: serde::Deserializer<'de>,
     {
-        String::deserialize(deserializer).map(|vs| vs.into())
-    }
-}
-
-/// Represents the known high-level content containers in Microformats.
-#[derive(Clone, Debug, PartialEq)]
-pub enum Kind {
-    /// https://microformats.org/wiki/h-entry
-    Entry,
-
-    /// https://microformats.org/wiki/h-event
-    Event,
-
-    /// https://microformats.org/wiki/h-card
-    Card,
-
-    /// https://microformats.org/wiki/h-review
-    Review,
-
-    /// https://microformats.org/wiki/h-resume
-    Resume,
-
-    /// https://microformats.org/wiki/h-app
-    /// https://indieweb.org/h-app
-    Application,
-
-    Other(String),
-}
-
-impl ToString for Kind {
-    /// Returns this kind as its known Microformat2 representation.
-    fn to_string(&self) -> String {
-        match self {
-            Kind::Entry => "h-entry".to_string(),
-            Kind::Event => "h-event".to_string(),
-            Kind::Card => "h-card".to_string(),
-            Kind::Review => "h-review".to_string(),
-            Kind::Resume => "h-resume".to_string(),
-            Kind::Application => "h-app".to_string(),
-            Kind::Other(other) => format!("h-x-{}", other),
-        }
-    }
-}
-
-impl FromStr for Kind {
-    type Err = std::convert::Infallible;
-    /// Parses a string value that could a potential Microformats2 kind representation into its
-    /// kind. Even if it doesn't recognize the type, it'll take the permissive approach by
-    /// Microformats2 and convert it into an entry.
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        use Kind::*;
-        Ok(match s {
-            "event" => Event,
-            "h-event" => Event,
-            "h-x-app" => Application,
-            "app" => Application,
-            "h-app" => Application,
-            "card" => Card,
-            "h-card" => Card,
-            "resume" => Resume,
-            "h-resume" => Resume,
-            "h-entry" => Entry,
-            "entry" => Entry,
-            other => Other(other.to_string()),
-        })
-    }
-}
-
-impl<'de> Deserialize<'de> for Kind {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: serde::Deserializer<'de>,
-    {
-        let s = dbg!(String::deserialize(deserializer)?);
-        Ok(Kind::from_str(&s).unwrap_or_default())
-    }
-}
-
-impl Serialize for Kind {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: serde::Serializer,
-    {
-        self.to_string().serialize(serializer)
-    }
-}
-
-impl Default for Kind {
-    fn default() -> Kind {
-        Self::Entry
+        String::deserialize(deserializer)
+            .and_then(|vs| Self::from_str(&vs).map_err(de::Error::custom))
     }
 }
 
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
 #[serde(untagged, rename_all = "kebab-case")]
-pub enum Channel {
+pub enum ChannelForm {
     Expanded {
         uid: String,
         name: String,
@@ -255,13 +167,13 @@ pub enum Channel {
     Simple(String),
 }
 
-impl Default for Channel {
+impl Default for ChannelForm {
     fn default() -> Self {
         Self::Simple("default".to_owned())
     }
 }
 
-impl Channel {
+impl ChannelForm {
     pub fn name(&self) -> String {
         match self {
             Self::Simple(name) => name.to_owned(),
@@ -277,8 +189,8 @@ impl Channel {
     }
 }
 
-impl FromStr for Channel {
-    type Err = std::convert::Infallible;
+impl FromStr for ChannelForm {
+    type Err = crate::Error;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         Ok(Self::Simple(s.to_string()))
@@ -287,7 +199,7 @@ impl FromStr for Channel {
 
 /// Represents the normalized form of pagination in Micropub queries.
 #[derive(Deserialize, Serialize, Clone, Debug, PartialEq, Default)]
-#[serde(rename_all = "kebab-case")]
+#[serde(rename_all = "kebab-case", deny_unknown_fields)]
 pub struct Pagination {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub limit: Option<String>,
@@ -334,7 +246,7 @@ pub struct ConfigurationResponse {
     pub q: Vec<String>,
 
     #[serde(default)]
-    pub channels: Vec<Channel>,
+    pub channels: Vec<ChannelForm>,
 
     #[serde(default)]
     pub media_endpoint: Option<url::Url>,
@@ -347,29 +259,26 @@ pub struct ConfigurationResponse {
 }
 
 #[derive(Serialize, Deserialize, PartialEq, Debug, Clone)]
-#[serde(rename_all = "kebab-case", deny_unknown_fields)]
+#[serde(rename_all = "kebab-case")]
 pub struct SourceResponse {
-    /// If provided/filled, this is the hinted post type.
-    #[serde(default)]
+    /// The hinted post type.
+    #[serde(default, rename = "post-type", with = "as_string_or_list")]
     pub post_type: Vec<Type>,
 
-    /// This is the kind of container object this Microformats Item is.
-    #[serde(rename = "type")]
-    pub kind: Vec<Kind>,
-
-    pub properties: Properties,
+    #[serde(flatten)]
+    pub item: Item,
 }
 
 #[derive(PartialEq, Debug, Clone, Serialize, Deserialize)]
 #[serde(rename_all = "kebab-case", deny_unknown_fields)]
 pub struct ChannelResponse {
-    pub channels: Vec<Channel>,
+    pub channels: Vec<ChannelForm>,
 }
 
 /// Represents the response for a query.
 // FIXME: Add the representation of syndication targets.
 #[derive(Serialize, Deserialize, PartialEq, Debug, Clone)]
-#[serde(untagged, rename_all = "kebab-case", deny_unknown_fields)]
+#[serde(untagged, rename_all = "kebab-case")]
 pub enum QueryResponse {
     Source(SourceResponse),
     Channel(ChannelResponse),
@@ -377,9 +286,101 @@ pub enum QueryResponse {
     Paginated(PaginatedResponse),
 }
 
-#[derive(Serialize, Deserialize)]
+/// Used to convert a value or a list of values into a normalized list of values.
+pub mod as_string_or_list {
+    use super::*;
+
+    pub fn deserialize<'de, Item, D>(deserializer: D) -> Result<Vec<Item>, D::Error>
+    where
+        Item: Deserialize<'de> + FromStr<Err = crate::Error> + fmt::Debug,
+        D: Deserializer<'de>,
+    {
+        struct CocereIntoList;
+
+        impl<'de> Visitor<'de> for CocereIntoList {
+            type Value = Vec<String>;
+
+            fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                formatter.write_str(
+                    "expecting no value, a string or a list of values, all to be made into a list",
+                )
+            }
+
+            fn visit_str<E>(self, v: &str) -> Result<Self::Value, E>
+            where
+                E: de::Error,
+            {
+                if v.is_empty() {
+                    Ok(Vec::default())
+                } else {
+                    Ok(vec![v.to_string()])
+                }
+            }
+
+            fn visit_seq<A>(self, mut seq: A) -> Result<Self::Value, A::Error>
+            where
+                A: de::SeqAccess<'de>,
+            {
+                let mut values = Vec::with_capacity(seq.size_hint().unwrap_or_default());
+
+                while let Some(value) = seq.next_element()? {
+                    values.push(value);
+                }
+
+                Ok(values)
+            }
+        }
+
+        deserializer
+            .deserialize_any(CocereIntoList)
+            .and_then(|strings| {
+                strings.into_iter().try_fold(
+                    Vec::default(),
+                    |mut acc, string| match Item::from_str(&string) {
+                        Ok(v) => {
+                            acc.push(v);
+                            Ok(acc)
+                        }
+                        Err(e) => Err(de::Error::custom(e)),
+                    },
+                )
+            })
+    }
+
+    pub fn serialize<Item, S>(list: &Vec<Item>, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        Item: ToString + Serialize,
+        S: serde::Serializer,
+    {
+        if let Some(value) = list.first().filter(|_| list.len() == 1) {
+            serializer.serialize_str(value.to_string().as_str())
+        } else {
+            let mut seq = serializer.serialize_seq(Some(list.len()))?;
+
+            for v in list {
+                seq.serialize_element(v)?;
+            }
+
+            seq.end()
+        }
+    }
+}
+
+#[derive(Serialize, Deserialize, Debug)]
+#[serde(rename_all = "kebab-case")]
+pub struct Query {
+    /// A representation of the base querying components.
+    #[serde(flatten, rename = "q")]
+    pub kind: QueryKind,
+
+    /// Pagination options to modify where in the list to look
+    #[serde(flatten, default)]
+    pub pagination: Pagination,
+}
+
+#[derive(Serialize, Deserialize, Debug)]
 #[serde(rename_all = "kebab-case", tag = "q")]
-pub enum Query {
+pub enum QueryKind {
     /// Pulls the configuration of the Micropub server.
     #[serde(rename = "config")]
     Configuration,
@@ -387,34 +388,44 @@ pub enum Query {
     /// Represents a call to `?q=source` for post information
     #[serde(rename_all = "kebab-case")]
     Source {
-        #[serde(skip_serializing_if = "Option::is_none")]
+        #[serde(skip_serializing_if = "Option::is_none", default)]
         url: Option<url::Url>,
 
-        #[serde(skip_serializing_if = "Option::is_none")]
-        post_type: Option<Type>,
+        /// Represents one or many post types to ask for when filtering this query.
+        #[serde(
+            with = "as_string_or_list",
+            default,
+            skip_serializing_if = "Vec::is_empty"
+        )]
+        post_type: Vec<Type>,
 
-        #[serde(skip_serializing_if = "Option::is_none")]
-        channel: Option<Channel>,
-
-        #[serde(flatten)]
-        pagination: Pagination,
+        /// Represents one or many channels to ask for when filtering this query.
+        #[serde(skip_serializing_if = "Vec::is_empty", default)]
+        channel: Vec<ChannelForm>,
     },
 
-    /// Represents a call to `?q=channel` to fetch all channels.
+    /// Represents a call to `?q=channel` to fetch channels.
     /// https://github.com/indieweb/micropub-extensions/issues/40
     #[serde(rename_all = "kebab-case")]
-    Channel {
-        #[serde(flatten)]
-        pagination: Pagination,
-    },
+    Channel,
 
+    /// Represents a call to `?q=syndicate-to` to fetch syndication targets.
     #[serde(rename_all = "kebab-case")]
     SyndicateTo {
-        #[serde(default, skip_serializing_if = "Option::is_none")]
-        post_type: Option<Type>,
+        /// Represents a single or many post types to ask for when filtering this query.
+        #[serde(with = "as_string_or_list", skip_serializing_if = "Vec::is_empty")]
+        post_type: Vec<Type>,
+    },
 
-        #[serde(flatten)]
-        pagination: Pagination,
+    /// Represents a call to `?q=rel` to fetch relationship links.
+    #[serde(alias = "rel")]
+    Relation {
+        /// The relations to explicitly look for.
+        #[serde(default, skip_serializing_if = "Vec::is_empty")]
+        rel: Vec<String>,
+
+        /// The identity to look up relationships for.
+        url: url::Url,
     },
 }
 
@@ -426,18 +437,15 @@ impl Query {
         access_token: &AccessToken,
     ) -> Result<ResponseValue<QueryResponse>, crate::Error> {
         let mut url = endpoint.clone();
-        let params = serde_json::to_value(self.clone())
-            .map_err(crate::Error::JSON)?
-            .as_object()
-            .cloned()
-            .unwrap_or_default();
-        url.query_pairs_mut()
-            .extend_pairs(params.into_iter().map(|(key, value)| {
-                (
-                    key,
-                    value.as_str().map(|s| s.to_string()).unwrap_or_default(),
-                )
-            }));
+        let self_string = serde_qs::to_string(&self).map_err(crate::Error::Qs)?;
+        if url.query().is_none() {
+            url.set_query(Some(self_string.as_str()));
+        } else {
+            let self_params = url::form_urlencoded::parse(self_string.as_bytes());
+            url.query_pairs_mut().extend_pairs(self_params);
+        }
+
+        log::trace!("Sending request to {:?}", url);
         let req = client::Request {
             headers: client::Headers::from_iter(vec![
                 ("accept", "application/json"),
@@ -451,40 +459,72 @@ impl Query {
             url,
         };
 
-        req.send("get").and_then(client::Response::into_json)
+        req.send("GET").and_then(client::Response::into_json)
+    }
+}
+
+impl ToString for Query {
+    fn to_string(&self) -> String {
+        serde_qs::to_string(self).unwrap_or_default()
     }
 }
 
 #[cfg(test)]
 #[allow(non_snake_case)]
 mod tests {
+    use microformats::types::{Class, KnownClass};
     use serde_json::json;
 
     use super::*;
 
     #[test]
-    fn query_send_formats_query_result_correctly_for_a_single_url() {}
-
-    #[test]
-    fn query_send_formats_query_result_correctly_for_post_type_list() {}
-
-    #[test]
-    fn query_send_looks_up_configuration() {}
-
-    #[test]
-    fn query_send_looks_up_channels() {}
-
-    #[test]
-    fn pagination_from_string() {}
-
-    #[test]
-    fn query_response_for_pagination() {}
+    fn query_from_qs_str() {
+        assert_eq!(
+            Some(vec![Type::Note, Type::Read]),
+            serde_qs::from_str::<Query>("q=source&post-type[]=note&post-type[]=read")
+                .ok()
+                .map(|q| q.kind)
+                .map(|k| match k {
+                    QueryKind::Source { post_type, .. } => post_type,
+                    _ => vec![],
+                }),
+            "provides deserialization of the source query with multiple post type asked for"
+        );
+        assert_eq!(
+            Some(vec![Type::Video, Type::Read]),
+            serde_qs::Config::new(1, false)
+                .deserialize_str::<Query>(
+                    "q=source&post-type%5B0%5D=video&post-type%5B1%5D=read&limit=5"
+                )
+                .ok()
+                .map(|q| q.kind)
+                .map(|k| match k {
+                    QueryKind::Source { post_type, .. } => post_type,
+                    _ => vec![],
+                }),
+            "provides deserialization of the source query with multiple post type asked for"
+        );
+        assert_eq!(
+            None,
+            serde_qs::from_str::<Query>("q=source&post-type=")
+                .err()
+                .map(|e| e.to_string()),
+            "ignores if the post type value is 'empty'"
+        );
+        assert_eq!(
+            None,
+            serde_qs::from_str::<Query>("q=config")
+                .err()
+                .map(|e| e.to_string()),
+            "provides deserialization of the config query"
+        );
+    }
 
     #[test]
     fn query_response_for_channels() {
         assert_eq!(
             Some(QueryResponse::Channel(ChannelResponse {
-                channels: vec![Channel::Expanded {
+                channels: vec![ChannelForm::Expanded {
                     uid: "magic".to_string(),
                     name: "Magic".to_string(),
                     properties: Properties::try_from(json!({"grr": "bark"})).unwrap()
@@ -506,7 +546,7 @@ mod tests {
         assert_eq!(
             Some(QueryResponse::Configuration(ConfigurationResponse {
                 q: vec!["channels".to_owned()],
-                channels: vec![Channel::Simple("all".to_owned())],
+                channels: vec![ChannelForm::Simple("all".to_owned())],
                 media_endpoint: None,
                 post_types: vec![PostType::Simple(Type::Note)],
                 syndicate_to: vec![]
@@ -520,18 +560,66 @@ mod tests {
 
     #[test]
     fn query_response_for_source() {
+        crate::test::init();
+        let mut item = Item::default();
+        item.r#type = vec![Class::Known(KnownClass::Entry)];
         assert_eq!(
-            Some(QueryResponse::Source(SourceResponse {
-                post_type: vec![Type::Note],
-                kind: vec![Kind::Entry],
-                properties: Properties::default()
+            Ok(QueryResponse::Source(SourceResponse {
+                post_type: vec![],
+                item: item.clone()
             })),
             serde_json::from_value(json!({
                 "type": ["h-entry"],
                 "properties": {}
             }))
-            .ok()
+            .map_err(|e| e.to_string())
         );
+
+        assert_eq!(
+            serde_json::from_value::<QueryResponse>(json!(
+            {
+                "post-type": [
+                    "article"
+                ],
+                "properties": {
+                    "audience": [],
+                    "category": [],
+                    "channel": [
+                        "all"
+                    ],
+                    "content": {
+                        "html": "<p>well-here-we-go</p>"
+                    },
+                    "name": "magic-omg",
+                    "post-status": [
+                        "published"
+                    ],
+                    "published": [
+                        "2022-02-12T23:22:27+00:00"
+                    ],
+                    "slug": [
+                        "Gzg043ii"
+                    ],
+                    "syndication": [],
+                    "updated": [
+                        "2022-02-12T23:22:27+00:00"
+                    ],
+                    "url": [
+                        "http://localhost:3112/Gzg043ii"
+                    ],
+                    "visibility": [
+                        "public"
+                    ]
+                },
+                "type": [
+                    "h-entry"
+                ]
+            }
+                    ))
+            .map_err(|e| e.to_string())
+            .err(),
+            None,
+        )
     }
 
     #[test]
